#!/bin/bash
#
#  Copyright 2019 poplite <poplite.xyz@gmail.com>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#

set -e

VERSION="1.5"

PROGRAM_NAME="libtorrent-rasterbar"
SCRIPT_NAME=$0

# Read version from argument
DEB_VERSION=$1
SUB_VERSION=${2:-"1"}

# Currently supported version
CURR_VERSION="1.2.13"

# List for options
OPTS="OPT_FULLNAME OPT_EMAIL OPT_DISTROS OPT_PPA_URL OPT_PPA_KEY"

# List for extended options
EXT_OPTS="OPT_NEW_RELEASE OPT_UPLOAD_ORIG_PACKAGE OPT_UPLOAD_CHANGES_FILE"

###### Options starts ######

# Maintainer information
OPT_FULLNAME=${OPT_FULLNAME:-"poplite"}
OPT_EMAIL=${OPT_EMAIL:-"poplite.xyz@gmail.com"}

# Supported distributions
OPT_DISTROS=${OPT_DISTROS:-"bionic focal impish jammy"}

# PPA infomation
OPT_PPA_URL=${OPT_PPA_URL:-"ppa:poplite/qbittorrent-enhanced"}
#OPT_PPA_URL=${OPT_PPA_URL:-"ppa:poplite/testing"}
OPT_PPA_KEY=${OPT_PPA_KEY:-"F1B89752"}

# Extended options
OPT_NEW_RELEASE=${OPT_NEW_RELEASE:-"yes"}

OPT_UPLOAD_ORIG_PACKAGE=${OPT_UPLOAD_ORIG_PACKAGE:-"once"}

OPT_UPLOAD_CHANGES_FILE=${OPT_UPLOAD_CHANGES_FILE:-"no"}

###### Options ends ######

# Maintainer information, needed by dch
export DEBFULLNAME="$OPT_FULLNAME"
export DEBEMAIL="$OPT_EMAIL"

# distribution name placeholder, used by sed
DIST_PLACEHOLDER="DISTRO"

# Options for dch
DCH_OPT="--package ${PROGRAM_NAME} --distribution ${DIST_PLACEHOLDER} --force-distribution --force-bad-version"

# Options for debuild
DEBUILD_OPT="-d -pgpg2 -k${OPT_PPA_KEY}"
# Disable uploading original package
if [[ "$OPT_UPLOAD_ORIG_PACKAGE" == "no" ]]
then
    DEBUILD_OPT="${DEBUILD_OPT} -sd"
fi

SOURCE_DIR="${PROGRAM_NAME}-${DEB_VERSION}"
TARBALL="${PROGRAM_NAME}_${DEB_VERSION}.orig.tar.gz"

BOLD_GREEN="\e[1;32m"
CLEAR_CLR="\e[0m"

pushd() {
    command pushd "$@" > /dev/null
}

popd() {
    command popd "$@" > /dev/null
}

echo_clr() {
    echo -e "${BOLD_GREEN}\n$1${CLEAR_CLR}"
}

usage() {
    echo -e "makePPA.sh for ${PROGRAM_NAME}"
    echo -e "Usage: ${SCRIPT_NAME} version [sub-version]\n"
    echo -e "version     - Main version (Example: 1.1.13)"
    echo -e "sub-version - Sub version  (Default: 1)"
}

print_info() {
    echo -e "Release version:" "${BOLD_GREEN}${DEB_VERSION}-${SUB_VERSION}${CLEAR_CLR}"
    echo -e "Maintainer     :" "${BOLD_GREEN}${DEBFULLNAME} <${DEBEMAIL}>${CLEAR_CLR}"
    echo -e "Launchpad PPA  :" "${BOLD_GREEN}${OPT_PPA_URL}${CLEAR_CLR}"
    echo -e "Release distros:" "${BOLD_GREEN}${OPT_DISTROS}${CLEAR_CLR}"
    echo -e "Debuild options:" "${BOLD_GREEN}${DEBUILD_OPT}${CLEAR_CLR}"
    # Print extended options
    echo -ne "Extends options: "
    for opt in $EXT_OPTS;
    do
        echo -ne "${opt}=${BOLD_GREEN}${!opt}${CLEAR_CLR} "
    done
    echo -e ""
}

# Show help
if [ -z "${DEB_VERSION}" ] || [ "$1" == "-h" ] || [ "$1" == "--help" ];
then
    usage
    exit 1
fi

print_info

# 1. Create a new directory
[ -d "${DEB_VERSION}" ] && rm -rf "${DEB_VERSION}"
mkdir -p "${DEB_VERSION}"
cd "${DEB_VERSION}"

# 2. Download tarball from Github
echo_clr "Downloading tarball..."
mkdir tmp
uscan --package "${PROGRAM_NAME}" --watchfile ../debian/watch                   \
      --upstream-version "${DEB_VERSION}" --download-version "${DEB_VERSION}"   \
      --overwrite-download --no-signature --rename --destdir ./tmp/ --verbose

# 3. Extract the tarball and copy debian directory
pushd tmp
tar -xzf "${TARBALL}"
cp -R ../../debian "${SOURCE_DIR}"

# 4. Add new release to changelog
if [[ "$OPT_NEW_RELEASE" == "yes" ]]
then
    echo_clr "Add new release to changelog"
    cd "${SOURCE_DIR}"
    dch ${DCH_OPT} \
        --newversion "${DEB_VERSION}-${SUB_VERSION}ppa1~${DIST_PLACEHOLDER}1" \
        "New upstream version ${DEB_VERSION}" 2>/dev/null
    dch -e
    head -n 5 debian/changelog
fi

popd # tmp/

for DISTRO in ${OPT_DISTROS}; 
do
    echo_clr "${DISTRO}: Copying source tree..."
    cp -R tmp "${DISTRO}"
    pushd "${DISTRO}"

    cd "${SOURCE_DIR}"
    
    # 5. Apply distro-specified patches
    DISTRO_PREBUILD_PATCHES_DIR="../../../distro/${DISTRO}/prebuild-patches"
    DISTRO_PATCHES_DIR="../../../distro/${DISTRO}/patches"
    if [[ -s "${DISTRO_PREBUILD_PATCHES_DIR}/series" ]]
    then
      echo_clr "${DISTRO}: Applying distro-specified prebuild patches..."
      for patch in $(grep -v \# "${DISTRO_PREBUILD_PATCHES_DIR}/series");
      do
          patch -p1 < "${DISTRO_PREBUILD_PATCHES_DIR}/${patch}"
      done
    fi
    if [[ -s "${DISTRO_PATCHES_DIR}/series" ]]
    then
      echo_clr "${DISTRO}: Applying distro-specified patches..."
      for patch in $(grep -v \# "${DISTRO_PATCHES_DIR}/series");
      do
          cp "${DISTRO_PATCHES_DIR}/${patch}" debian/patches
          echo $patch >> debian/patches/series
      done
    fi

    # 6. Signing changes files
    echo_clr "${DISTRO}: signing changes files..."
    sed -i "s/${DIST_PLACEHOLDER}/${DISTRO}/g" debian/changelog
    debuild -S ${DEBUILD_OPT}

    # 7. Upload signed changes files to Launchpad
    if [[ "$OPT_UPLOAD_CHANGES_FILE" == "yes" ]]
    then
        echo_clr "${DISTRO}: Uploading changes files..."
        cd .. # ${DISTRO}/
        dput "${OPT_PPA_URL}" *.changes
    fi

    popd # ${DISTRO}/
    
    # Disable uploading original package
    if [[ "$OPT_UPLOAD_ORIG_PACKAGE" == "once" ]]
    then
        DEBUILD_OPT="${DEBUILD_OPT} -sd"
        OPT_UPLOAD_ORIG_PACKAGE="no"
    fi
done 

echo_clr "Done."

exit 0
